(* This file was generated automatically by wayland-scanner-ocaml *)

open struct
  module Proxy = Wayland.Proxy
  module Iface_reg = Wayland.Iface_reg
  module Metadata = Wayland.Metadata
end

module Xdg_wm_base = struct
  type t = [`Xdg_wm_base]
  type _ Metadata.ty += T : [`Xdg_wm_base] Metadata.ty
  type versions = [`V1 | `V2 | `V3]
  let interface = "xdg_wm_base"
  let version = 3l
  
  module Error = struct
    type t =
      | Role : t
      | Defunct_surfaces : t
      | Not_the_topmost_popup : t
      | Invalid_popup_parent : t
      | Invalid_surface_state : t
      | Invalid_positioner : t
    
    let to_int32 = function
      | Role -> 0l
      | Defunct_surfaces -> 1l
      | Not_the_topmost_popup -> 2l
      | Invalid_popup_parent -> 3l
      | Invalid_surface_state -> 4l
      | Invalid_positioner -> 5l
    
    let of_int32 = function
      | 0l -> Role
      | 1l -> Defunct_surfaces
      | 2l -> Not_the_topmost_popup
      | 3l -> Invalid_popup_parent
      | 4l -> Invalid_surface_state
      | 5l -> Invalid_positioner
      | x -> Fmt.failwith "Invalid error enum value %ld" x
  end
  
  let requests = function
    | 0 -> "destroy", []
    | 1 -> "create_positioner", ["id", `New_ID (Some "xdg_positioner")]
    | 2 -> "get_xdg_surface", ["id", `New_ID (Some "xdg_surface"); "surface", `Object (Some "wl_surface")]
    | 3 -> "pong", ["serial", `Uint]
    | i -> Proxy.unknown_request i, []
  
  let events = function
    | 0 -> "ping", ["serial", `Uint]
    | i -> Proxy.unknown_event i, []
  
end
let () = Iface_reg.register (module Xdg_wm_base)

module Xdg_positioner = struct
  type t = [`Xdg_positioner]
  type _ Metadata.ty += T : [`Xdg_positioner] Metadata.ty
  type versions = [`V1 | `V2 | `V3]
  let interface = "xdg_positioner"
  let version = 3l
  
  module Error = struct
    type t =
      | Invalid_input : t
    
    let to_int32 = function
      | Invalid_input -> 0l
    
    let of_int32 = function
      | 0l -> Invalid_input
      | x -> Fmt.failwith "Invalid error enum value %ld" x
  end
  
  
  module Anchor = struct
    type t =
      | None : t
      | Top : t
      | Bottom : t
      | Left : t
      | Right : t
      | Top_left : t
      | Bottom_left : t
      | Top_right : t
      | Bottom_right : t
    
    let to_int32 = function
      | None -> 0l
      | Top -> 1l
      | Bottom -> 2l
      | Left -> 3l
      | Right -> 4l
      | Top_left -> 5l
      | Bottom_left -> 6l
      | Top_right -> 7l
      | Bottom_right -> 8l
    
    let of_int32 = function
      | 0l -> None
      | 1l -> Top
      | 2l -> Bottom
      | 3l -> Left
      | 4l -> Right
      | 5l -> Top_left
      | 6l -> Bottom_left
      | 7l -> Top_right
      | 8l -> Bottom_right
      | x -> Fmt.failwith "Invalid anchor enum value %ld" x
  end
  
  
  module Gravity = struct
    type t =
      | None : t
      | Top : t
      | Bottom : t
      | Left : t
      | Right : t
      | Top_left : t
      | Bottom_left : t
      | Top_right : t
      | Bottom_right : t
    
    let to_int32 = function
      | None -> 0l
      | Top -> 1l
      | Bottom -> 2l
      | Left -> 3l
      | Right -> 4l
      | Top_left -> 5l
      | Bottom_left -> 6l
      | Top_right -> 7l
      | Bottom_right -> 8l
    
    let of_int32 = function
      | 0l -> None
      | 1l -> Top
      | 2l -> Bottom
      | 3l -> Left
      | 4l -> Right
      | 5l -> Top_left
      | 6l -> Bottom_left
      | 7l -> Top_right
      | 8l -> Bottom_right
      | x -> Fmt.failwith "Invalid gravity enum value %ld" x
  end
  
  
  (** Constraint adjustments.
      
      The constraint adjustment value define ways the compositor will adjust
      the position of the surface, if the unadjusted position would result
      in the surface being partly constrained.
      
      Whether a surface is considered 'constrained' is left to the compositor
      to determine. For example, the surface may be partly outside the
      compositor's defined 'work area', thus necessitating the child surface's
      position be adjusted until it is entirely inside the work area.
      
      The adjustments can be combined, according to a defined precedence: 1)
      Flip, 2) Slide, 3) Resize. *)
  module Constraint_adjustment = struct
    type t = int32
    
    (** Don't move the child surface when constrained.
        
        Don't alter the surface position even if it is constrained on some
        axis, for example partially outside the edge of an output. *)
    let none = 0l
    
    (** Move along the x axis until unconstrained.
        
        Slide the surface along the x axis until it is no longer constrained.
        
        First try to slide towards the direction of the gravity on the x axis
        until either the edge in the opposite direction of the gravity is
        unconstrained or the edge in the direction of the gravity is
        constrained.
        
        Then try to slide towards the opposite direction of the gravity on the
        x axis until either the edge in the direction of the gravity is
        unconstrained or the edge in the opposite direction of the gravity is
        constrained. *)
    let slide_x = 1l
    
    (** Move along the y axis until unconstrained.
        
        Slide the surface along the y axis until it is no longer constrained.
        
        First try to slide towards the direction of the gravity on the y axis
        until either the edge in the opposite direction of the gravity is
        unconstrained or the edge in the direction of the gravity is
        constrained.
        
        Then try to slide towards the opposite direction of the gravity on the
        y axis until either the edge in the direction of the gravity is
        unconstrained or the edge in the opposite direction of the gravity is
        constrained. *)
    let slide_y = 2l
    
    (** Invert the anchor and gravity on the x axis.
        
        Invert the anchor and gravity on the x axis if the surface is
        constrained on the x axis. For example, if the left edge of the
        surface is constrained, the gravity is 'left' and the anchor is
        'left', change the gravity to 'right' and the anchor to 'right'.
        
        If the adjusted position also ends up being constrained, the resulting
        position of the flip_x adjustment will be the one before the
        adjustment. *)
    let flip_x = 4l
    
    (** Invert the anchor and gravity on the y axis.
        
        Invert the anchor and gravity on the y axis if the surface is
        constrained on the y axis. For example, if the bottom edge of the
        surface is constrained, the gravity is 'bottom' and the anchor is
        'bottom', change the gravity to 'top' and the anchor to 'top'.
        
        The adjusted position is calculated given the original anchor
        rectangle and offset, but with the new flipped anchor and gravity
        values.
        
        If the adjusted position also ends up being constrained, the resulting
        position of the flip_y adjustment will be the one before the
        adjustment. *)
    let flip_y = 8l
    
    (** Horizontally resize the surface.
        
        Resize the surface horizontally so that it is completely
        unconstrained. *)
    let resize_x = 16l
    
    (** Vertically resize the surface.
        
        Resize the surface vertically so that it is completely unconstrained. *)
    let resize_y = 32l
    
    let to_int32 = Fun.id
    let of_int32 = Fun.id
  end
  
  let requests = function
    | 0 -> "destroy", []
    | 1 -> "set_size", ["width", `Int; "height", `Int]
    | 2 -> "set_anchor_rect", ["x", `Int; "y", `Int; "width", `Int; "height", `Int]
    | 3 -> "set_anchor", ["anchor", `Uint]
    | 4 -> "set_gravity", ["gravity", `Uint]
    | 5 -> "set_constraint_adjustment", ["constraint_adjustment", `Uint]
    | 6 -> "set_offset", ["x", `Int; "y", `Int]
    | 7 -> "set_reactive", []
    | 8 -> "set_parent_size", ["parent_width", `Int; "parent_height", `Int]
    | 9 -> "set_parent_configure", ["serial", `Uint]
    | i -> Proxy.unknown_request i, []
  
  let events = function
    | i -> Proxy.unknown_event i, []
  
end
let () = Iface_reg.register (module Xdg_positioner)

module Xdg_surface = struct
  type t = [`Xdg_surface]
  type _ Metadata.ty += T : [`Xdg_surface] Metadata.ty
  type versions = [`V1 | `V2 | `V3]
  let interface = "xdg_surface"
  let version = 3l
  
  module Error = struct
    type t =
      | Not_constructed : t
      | Already_constructed : t
      | Unconfigured_buffer : t
    
    let to_int32 = function
      | Not_constructed -> 1l
      | Already_constructed -> 2l
      | Unconfigured_buffer -> 3l
    
    let of_int32 = function
      | 1l -> Not_constructed
      | 2l -> Already_constructed
      | 3l -> Unconfigured_buffer
      | x -> Fmt.failwith "Invalid error enum value %ld" x
  end
  
  let requests = function
    | 0 -> "destroy", []
    | 1 -> "get_toplevel", ["id", `New_ID (Some "xdg_toplevel")]
    | 2 -> "get_popup", ["id", `New_ID (Some "xdg_popup"); "parent", `Object (Some "xdg_surface");
                         "positioner", `Object (Some "xdg_positioner")]
    | 3 -> "set_window_geometry", ["x", `Int; "y", `Int; "width", `Int; "height", `Int]
    | 4 -> "ack_configure", ["serial", `Uint]
    | i -> Proxy.unknown_request i, []
  
  let events = function
    | 0 -> "configure", ["serial", `Uint]
    | i -> Proxy.unknown_event i, []
  
end
let () = Iface_reg.register (module Xdg_surface)

module Xdg_toplevel = struct
  type t = [`Xdg_toplevel]
  type _ Metadata.ty += T : [`Xdg_toplevel] Metadata.ty
  type versions = [`V1 | `V2 | `V3]
  let interface = "xdg_toplevel"
  let version = 3l
  
  (** Edge values for resizing.
      
      These values are used to indicate which edge of a surface
      is being dragged in a resize operation. *)
  module Resize_edge = struct
    type t =
      | None : t
      | Top : t
      | Bottom : t
      | Left : t
      | Top_left : t
      | Bottom_left : t
      | Right : t
      | Top_right : t
      | Bottom_right : t
    
    let to_int32 = function
      | None -> 0l
      | Top -> 1l
      | Bottom -> 2l
      | Left -> 4l
      | Top_left -> 5l
      | Bottom_left -> 6l
      | Right -> 8l
      | Top_right -> 9l
      | Bottom_right -> 10l
    
    let of_int32 = function
      | 0l -> None
      | 1l -> Top
      | 2l -> Bottom
      | 4l -> Left
      | 5l -> Top_left
      | 6l -> Bottom_left
      | 8l -> Right
      | 9l -> Top_right
      | 10l -> Bottom_right
      | x -> Fmt.failwith "Invalid resize_edge enum value %ld" x
  end
  
  
  (** Types of state on the surface.
      
      The different state values used on the surface. This is designed for
      state values like maximized, fullscreen. It is paired with the
      configure event to ensure that both the client and the compositor
      setting the state can be synchronized.
      
      States set in this way are double-buffered. They will get applied on
      the next commit. *)
  module State = struct
    type t =
      | Maximized : t
        (** The surface is maximized.
            
            The surface is maximized. The window geometry specified in the configure
            event must be obeyed by the client.
            
            The client should draw without shadow or other
            decoration outside of the window geometry. *)
      | Fullscreen : t
        (** The surface is fullscreen.
            
            The surface is fullscreen. The window geometry specified in the
            configure event is a maximum; the client cannot resize beyond it. For
            a surface to cover the whole fullscreened area, the geometry
            dimensions must be obeyed by the client. For more details, see
            xdg_toplevel.set_fullscreen. *)
      | Resizing : t
        (** The surface is being resized.
            
            The surface is being resized. The window geometry specified in the
            configure event is a maximum; the client cannot resize beyond it.
            Clients that have aspect ratio or cell sizing configuration can use
            a smaller size, however. *)
      | Activated : t
        (** The surface is now activated.
            
            Client window decorations should be painted as if the window is
            active. Do not assume this means that the window actually has
            keyboard or pointer focus. *)
      | Tiled_left : t
        (** The surface is tiled.
            
            The window is currently in a tiled layout and the left edge is
            considered to be adjacent to another part of the tiling grid. *)
      | Tiled_right : t
        (** The surface is tiled.
            
            The window is currently in a tiled layout and the right edge is
            considered to be adjacent to another part of the tiling grid. *)
      | Tiled_top : t
        (** The surface is tiled.
            
            The window is currently in a tiled layout and the top edge is
            considered to be adjacent to another part of the tiling grid. *)
      | Tiled_bottom : t
        (** The surface is tiled.
            
            The window is currently in a tiled layout and the bottom edge is
            considered to be adjacent to another part of the tiling grid. *)
    
    let to_int32 = function
      | Maximized -> 1l
      | Fullscreen -> 2l
      | Resizing -> 3l
      | Activated -> 4l
      | Tiled_left -> 5l
      | Tiled_right -> 6l
      | Tiled_top -> 7l
      | Tiled_bottom -> 8l
    
    let of_int32 = function
      | 1l -> Maximized
      | 2l -> Fullscreen
      | 3l -> Resizing
      | 4l -> Activated
      | 5l -> Tiled_left
      | 6l -> Tiled_right
      | 7l -> Tiled_top
      | 8l -> Tiled_bottom
      | x -> Fmt.failwith "Invalid state enum value %ld" x
  end
  
  let requests = function
    | 0 -> "destroy", []
    | 1 -> "set_parent", ["parent", `Object (Some "xdg_toplevel")]
    | 2 -> "set_title", ["title", `String]
    | 3 -> "set_app_id", ["app_id", `String]
    | 4 -> "show_window_menu", ["seat", `Object (Some "wl_seat"); "serial", `Uint; "x", `Int; "y", `Int]
    | 5 -> "move", ["seat", `Object (Some "wl_seat"); "serial", `Uint]
    | 6 -> "resize", ["seat", `Object (Some "wl_seat"); "serial", `Uint; "edges", `Uint]
    | 7 -> "set_max_size", ["width", `Int; "height", `Int]
    | 8 -> "set_min_size", ["width", `Int; "height", `Int]
    | 9 -> "set_maximized", []
    | 10 -> "unset_maximized", []
    | 11 -> "set_fullscreen", ["output", `Object (Some "wl_output")]
    | 12 -> "unset_fullscreen", []
    | 13 -> "set_minimized", []
    | i -> Proxy.unknown_request i, []
  
  let events = function
    | 0 -> "configure", ["width", `Int; "height", `Int; "states", `Array]
    | 1 -> "close", []
    | i -> Proxy.unknown_event i, []
  
end
let () = Iface_reg.register (module Xdg_toplevel)

module Xdg_popup = struct
  type t = [`Xdg_popup]
  type _ Metadata.ty += T : [`Xdg_popup] Metadata.ty
  type versions = [`V1 | `V2 | `V3]
  let interface = "xdg_popup"
  let version = 3l
  
  module Error = struct
    type t =
      | Invalid_grab : t
    
    let to_int32 = function
      | Invalid_grab -> 0l
    
    let of_int32 = function
      | 0l -> Invalid_grab
      | x -> Fmt.failwith "Invalid error enum value %ld" x
  end
  
  let requests = function
    | 0 -> "destroy", []
    | 1 -> "grab", ["seat", `Object (Some "wl_seat"); "serial", `Uint]
    | 2 -> "reposition", ["positioner", `Object (Some "xdg_positioner"); "token", `Uint]
    | i -> Proxy.unknown_request i, []
  
  let events = function
    | 0 -> "configure", ["x", `Int; "y", `Int; "width", `Int; "height", `Int]
    | 1 -> "popup_done", []
    | 2 -> "repositioned", ["token", `Uint]
    | i -> Proxy.unknown_event i, []
  
end
let () = Iface_reg.register (module Xdg_popup)
